from .client import APIBase


class Chagebacks(APIBase):
    def create(self, invoice, amount, reason, reason_details=None):
        payload = {
            "invoice": invoice,
            "amount": amount,
            "reason": reason,
            "reason_details": reason_details
        }
        return self.send_request("POST", "chargebacks/", payload)

    def retrieve(self, chargeback_id=None):
        if chargeback_id:
            return self.details(chargeback_id)
        return self.send_request("GET", "chargebacks/", {})

    def details(self, chargeback_id=None):
        return self.send_request("GET", f"chargebacks/{chargeback_id}/", {})



from Crypto.PublicKey import RSA
import OpenSSL
from OpenSSL import crypto as OpenSSLCrypto


def generate_keys():
    """
    Returns a private and public key.

    Returns:
        tupple: values of generated private and public key
    """
    key = RSA.generate(2048)
    private_key = key.export_key('PEM')
    public_key = key.publickey().export_key('PEM')
    return private_key, public_key


def sign_message(private_key, message):
    """
    Sign message with the private key.

    Args:
        private_key (byte): Private key
        message (string): Message to sign

    Returns:
        string: Signed message
    """
    pkey = OpenSSLCrypto.load_privatekey(
        OpenSSLCrypto.FILETYPE_PEM, private_key, None)
    sign = OpenSSL.crypto.sign(pkey, message, "sha256")
    return sign.hex()



from .client import APIBase


class PaymentLinks(APIBase):
    def details(self, link_id):
        return self.send_request("GET", f"paymentlinks/{link_id}", None)

    def create(self, title, currency, amount=0, mobile_tarrif="BUSINESS-PAYS", card_tarrif="BUSINESS-PAYS", is_active=True, **kwargs):
        payload = kwargs
        payload["title"] = title
        payload["currency"] = currency
        payload["amount"] = amount
        payload["mobile_tarrif"] = mobile_tarrif
        payload["card_tarrif"] = card_tarrif
        payload["is_active"] = is_active
        return self.send_request("POST", "paymentlinks/", payload)

    def retrieve(self, link_id=None):
        if link_id:
            return self.details(link_id)
        return self.send_request("GET", "paymentlinks/", None)

    def deactivate(self):
        return self.send_request("GET", "paymentlinks/", None)

    def update(self):
        return self.send_request("GET", "paymentlinks/", None)



from .client import APIBase


class Customers(APIBase):
    def details(self, customer_id):
        return self.send_request("GET", f"customers/{customer_id}", None)

    def create(self, email, **kwargs):
        payload = kwargs
        payload["email"] = email
        return self.send_request("POST", "customers/", payload)

    def retrieve(self, customer_id=None):
        if customer_id:
            return self.details(customer_id)
        return self.send_request("GET", "customers/", None)

    def transactions(self, customer_id):
        return self.send_request("GET", f"customers/{customer_id}/transactions", None)



from .client import APIBase


class Devices(APIBase):
    def details(self, device_id):
        return self.send_request("GET", f"r'account-devices'/{device_id}", None)

    def retrieve(self, device_id=None):
        if device_id:
            return self.details(device_id)
        return self.send_request("GET", "r'account-devices'/", None)



from .client import APIBase


class Collect(APIBase):
    def checkout(self, email, amount, currency, **kwargs):
        """
        Generates payment checkout URL

        Args:
            email (string): Customer email
            amount (float): Total amount billed
            currency (string): Currency code (KES, USD, EUR, GBP)

        Returns:
            object: JSON dictionary with a checkout url field
        """
        method = kwargs.get("method")
        api_ref = kwargs.get("api_ref", "API Request")
        callback_url = kwargs.get("callback_url")
        comment = kwargs.get("comment")
        first_name = kwargs.get("first_name")
        last_name = kwargs.get("last_name")
        phone_number = kwargs.get("phone_number")
        mobile_tarrif = kwargs.get("mobile_tarrif", "BUSINESS-PAYS")
        card_tarrif = kwargs.get("card_tarrif", "BUSINESS-PAYS")
        payload = {
            "public_key": self.publishable_key,
            "currency": currency,
            "email": email,
            "amount": amount,
            "method": method,
            "api_ref": api_ref,
            "callback_url": callback_url,
            "comment": comment,
            "first_name": first_name,
            "last_name": last_name,
            "phone_number": phone_number,
            "mobile_tarrif": mobile_tarrif,
            "card_tarrif": card_tarrif,
            "version": "3.0.0"
        }
        print(payload)
        return self.send_request("POST", "checkout/", payload)

    def status(self, invoice_id, checkout_id=None, signature=None):
        """
        Check status of transaction/invoice

        Args:
            invoice_id (string): Invoice or tracking ID
            checkout_id (string, optional): [Checkout id for payments requests through checkout API]. Defaults to None.
            signature (string, optional): [JWT signature for payments requests through checkout API]. Defaults to None.

        Returns:
            object: JSON with transaction details
        """
        payload = {
            "invoice_id": invoice_id,
            "public_key": self.publishable_key,
        }
        if checkout_id and signature:
            payload = {
                "invoice_id": invoice_id,
                "signature": signature,
                "checkout_id": checkout_id,
            }
        return self.send_request("POST", "payment/status/", payload)

    def mpesa_stk_push(self, phone_number, amount, narrative, currency="KES", api_ref="API Request", name=None, email=None):
        payload = {
            "public_key": self.publishable_key,
            "currency": currency,
            "method": "M-PESA",
            "amount": amount,
            "phone_number": phone_number,
            "api_ref": api_ref,
            "name": name,
            "email": email
        }
        return self.send_request("POST", "payment/collection/", payload)

    def get_quote(self, amount, method, currency="KES", tarrif="BUSINESS-PAYS"):
        payload = {
            "public_key": self.publishable_key,
            "currency": currency,
            "method": method,
            "tarrif": tarrif
        }
        return self.send_request("POST", "payment/get_amount_estimate/", payload)



import requests

from .exceptions import (IntaSendBadRequest, IntaSendNotAllowed,
                         IntaSendServerError, IntaSendUnauthorized)


def get_service_url(service_endpoint, test=False):
    if test:
        return f"https://sandbox.intasend.com/api/v1/{service_endpoint}"
    return f"https://payment.intasend.com/api/v1/{service_endpoint}"


class APIBase(object):
    def __init__(self, **kwargs):
        """API helper defination."""
        self.token = kwargs.get("token")
        self.publishable_key = kwargs.get("publishable_key")
        self.private_key = kwargs.get("private_key")
        self.test = kwargs.get("test", False)
        if not self.token:
            raise Exception("Authentication token is required")
        super().__init__()

    def send_request(self, request_type, service_endpoint, payload, noauth=False):
        url = get_service_url(service_endpoint,  self.test)
        headers = self.get_headers()
        resp = requests.request(
            request_type, url, json=payload, headers=headers)
        if resp.status_code == 400:
            raise IntaSendBadRequest(resp.text)
        elif resp.status_code == 403:
            raise IntaSendNotAllowed(resp.text)
        elif resp.status_code == 500:
            raise IntaSendServerError(resp.text)
        elif resp.status_code == 401:
            raise IntaSendUnauthorized(resp.text)
        return resp.json()

    def get_headers(self):
        return {
            "Authorization": f"Bearer {self.token}"
        }



from .client import APIBase
from .utils import sign_message


class Transfer(APIBase):
    def send_money(self, device_id, provider, currency, transactions, callback_url=None, wallet_id=None):
        payload = {
            "device_id": device_id,
            "provider": provider,
            "currency": currency,
            "transactions": transactions,
            "callback_url": callback_url,
            "wallet_id": wallet_id
        }
        return self.send_request("POST", "send-money/initiate/", payload)

    def approve(self, payload):
        nonce = payload["nonce"]
        signed_nonce = sign_message(self.private_key.encode("utf-8"), nonce)
        payload["nonce"] = signed_nonce
        return self.send_request("POST", "send-money/approve/", payload)

    def status(self, tracking_id):
        payload = {
            "tracking_id": tracking_id
        }
        return self.send_request("POST", "send-money/status/", payload)

    def mpesa(self,  device_id, currency, transactions, callback_url=None, wallet_id=None):
        provider = "MPESA-B2C"
        return self.send_money(device_id, provider, currency, transactions, callback_url, wallet_id)

    def mpesa_b2b(self,  device_id, currency, transactions, callback_url=None, wallet_id=None):
        provider = "MPESA-B2B"
        return self.send_money(device_id, provider, currency, transactions, callback_url, wallet_id)

    def intasend(self, device_id, currency, transactions, callback_url=None, wallet_id=None):
        provider = "INTASEND"
        return self.send_money(device_id, provider, currency, transactions, callback_url, wallet_id)

    def bank(self, device_id, currency, transactions, callback_url=None, wallet_id=None):
        provider = "PESALINK"
        return self.send_money(device_id, provider, currency, transactions, callback_url, wallet_id)



from .client import APIBase
from .collections import Collect


class Wallet(APIBase):
    def __init__(self, **kwargs):
        """Wallets management service."""
        self.collect = Collect(**kwargs)
        super().__init__(**kwargs)

    def details(self, wallet_id):
        return self.send_request("GET", f"wallets/{wallet_id}", None)

    def create(self, currency, label, can_disburse=False):
        payload = {
            "wallet_type": "WORKING",
            "currency": currency,
            "label": label,
            "can_disburse": can_disburse
        }
        return self.send_request("POST", "wallets/", payload)

    def retrieve(self, wallet_id=None):
        if wallet_id:
            return self.details(wallet_id)
        return self.send_request("GET", "wallets/", None)

    def transactions(self, wallet_id):
        return self.send_request("GET", f"wallets/{wallet_id}/transactions", None)

    def intra_transfer(self, origin_id, destination_id, amount, narrative):
        payload = {
            "wallet_id": destination_id,
            "amount": amount,
            "narrative": narrative
        }
        return self.send_request("POST", f"wallets/{origin_id}/intra_transfer/", payload=payload)

    def fund(self, wallet_id, phone_number, email, amount, narrative, currency="KES", api_ref="API Request", name=None):
        return self.collect.mpesa_stk_push(phone_number=phone_number, email=email, amount=amount, narrative=narrative, currency=currency, api_ref=api_ref, name=name)



"""IntaSend Payments API for Python 3."""
from .collections import Collect
from .transfers import Transfer
from .wallets import Wallet
from .chargebacks import Chagebacks
from .payment_links import PaymentLinks
from .devices import Devices
from .customers import Customers


class APIService:
    def __init__(self, **kwargs):
        """API Services Initialization."""
        self.collect = Collect(**kwargs)
        self.transfer = Transfer(**kwargs)
        self.wallets = Wallet(**kwargs)
        self.chargebacks = Chagebacks(**kwargs)
        self.customers = Customers(**kwargs)
        self.devices = Devices(**kwargs)
        self.payment_links = PaymentLinks(**kwargs)



from setuptools import setup

setup(
    name='intasend-python',
    packages=['intasend'],
    version='1.0.2',
    license='MIT',
    description='Official Python SDK for IntaSend Payments Gateway API',
    author='Felix Cheruiyot',
    author_email='support@intasend.com',
    url='https://github.com/IntaSend/intasend-python',
    download_url='https://github.com/IntaSend/intasend-python/archive/v_1.0.2.tar.gz',
    keywords=['payments', 'mpesa', 'card payments', 'visa',
              'mastercard', 'payments kenya', 'intasend'],
    install_requires=[
        'requests',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ]
)
